@@ -21,13 +21,25 @@ from transformers_keras.modeling_bert import (
     BertPretrainedModel,
 )
 from transformers_keras.modeling_utils import complete_inputs
-from transformers_keras.question_answering.dataset import QuestionAnsweringDataset, QuestionAnsweringExample
-from transformers_keras.question_answering.models import AlbertForQuestionAnswering, BertForQuestionAnswering
-from transformers_keras.question_answering.tokenizer import (
-    QuestionAnsweringTokenizer,
-    QuestionAnsweringTokenizerForChinese,
+from transformers_keras.question_answering.callback import (
+    BaseMetricForQuestionAnswering,
+    ExactMatchForQuestionAnswering,
+    F1ForQuestionAnswering,
+)
+from transformers_keras.question_answering.dataset import (
+    QuestionAnsweringDataset,
+    QuestionAnsweringExample,
+    QuestionAnsweringXDataset,
+    QuestionAnsweringXExample,
+)
+from transformers_keras.question_answering.models import (
+    AlbertForQuestionAnswering,
+    AlbertForQuestionAnsweringX,
+    BertForQuestionAnswering,
+    BertForQuestionAnsweringX,
 )
 from transformers_keras.sentence_embedding.bert_embedding import BertForSentenceEmbedding
+from transformers_keras.sentence_embedding.callback import SpearmanForSentenceEmbedding
 from transformers_keras.sentence_embedding.simcse_dataset import SimCSEDataset, SimCSEExample
 from transformers_keras.sentence_embedding.simcse_models import HardNegativeSimCSE, SupervisedSimCSE, UnsupervisedSimCSE
 from transformers_keras.sequence_classification.dataset import (
@@ -38,6 +50,7 @@ from transformers_keras.sequence_classification.models import (
     AlbertForSequenceClassification,
     BertForSequenceClassification,
 )
+from transformers_keras.token_classification.callback import SeqEvalForTokenClassification
 from transformers_keras.token_classification.crf_models import (
     AlertCRFForTokenClassification,
     BertCRFForTokenClassification,
@@ -45,9 +58,8 @@ from transformers_keras.token_classification.crf_models import (
 )
 from transformers_keras.token_classification.dataset import TokenClassificationDataset, TokenClassificationExample
 from transformers_keras.token_classification.models import AlbertForTokenClassification, BertForTokenClassification
-from transformers_keras.token_classification.tokenizer import QuestionAnsweringTokenizerForChinese
 
 __name__ = "transformers_keras"
-__version__ = "0.4.3"
+__version__ = "0.4.4"
 
 logging.basicConfig(format="%(asctime)s %(levelname)7s %(filename)20s %(lineno)4d] %(message)s", level=logging.INFO)
